/*----------------------------------------------------------------------------
     "element.h" - element information classes
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2010 by A. Miyoshi, University of Tokyo
                   separated from "molecule.h": Oct.  30, 2003
                                        edited: Dec.  10, 2003
                                   last edited: Feb.  21, 2010
----------------------------------------------------------------------------*/

#ifndef _ELEMENT_H_
#define _ELEMENT_H_

#include "libfunc.h"
using namespace std;

#define FL_ELMTBL "elm_tbl.dat"         // element table

// ===== 'elementTable' and related classes ==============================

class element {
 public:                                         // --- data members ---
  string atomicSymbol;
  int valence;
  int hyperValence;
  int formOrder;
 public:                                     // --- member functions ---
  element();
  element(const element &el);
  void clear();
  int restoreFrom(ifstream &ifs);
  string canonSymbol();
};

class elementTable : public map<int, element> {
 protected:                                      // --- data members ---
  element none;
  map<int, string> aufbTbl;
  int aufbUpLimit;
 public:                                     // --- member functions ---
  elementTable();
  void init();
  int loadFrom(string flname, libPath &libp);
  int atmcNbrBySymb(const string &as);
  int isSupported(int an);
  const element &operator[](int an);
  int ionValence(int an, int chg, int &hyp);
  string canonSymbol(int an);
 protected:
  int aufbauValence(int an, string &orbit);
};
                                                      // --- globals ---
string rpsAtmAttr(int atmAttr);    // string repres. atom attribute
string rpsBndOrdr(int bndOrdr);    // string repres. bond order

// ===== 'atmcComp' and related classes ==================================

class atmcComp : public map<int, int> {
 public:                                     // --- member functions ---
  atmcComp &operator+=(atmcComp &ac);
  int operator==(atmcComp &ac);
  int operator!=(atmcComp &ac);
};
                                                      // --- globals ---
atmcComp operator+(atmcComp &acl, atmcComp &acr);

class atmcCompSorted : public map<int, pair<int, int> > {
 public:                                     // --- member functions ---
  void setFrom(atmcComp &ac);
};

// ===== global object of elementTable ===================================

extern elementTable elmt;

#endif
