/* -------------------------------------------------------------------------
     "joback.h" - transport property estimation according to Joback
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2011 by A. Miyoshi, University of Tokyo
                                       created: Jan.  25, 2011
                                   last edited: Jan.  25, 2011
------------------------------------------------------------------------- */

#ifndef _JOBACK_H_
#define _JOBACK_H_

#include "libfunc.h"
using namespace std;

#define FL_TRNGCM "trn_gcm.dat"         // group contribution values

// ===== Joback group contribution value library class =====================

class jobackValues {
 public:                                         // --- data members ---
  double tfpk, tbk, tck, pck, vck;
 public:                                     // --- member functions ---
  jobackValues();
  void clear();
};

class jobackGroupValues : public map<string, jobackValues> {
 public:                                     // --- member functions ---
  jobackGroupValues();
  int restoreFrom(ifstream &ifs);
};

class jobackAliases : public map<string, string> {
 public:                                     // --- member functions ---
  jobackAliases();
  int restoreFrom(ifstream &ifs);
};

class jobackGrpList : public map<string, int> {
 public:                                     // --- member functions ---
  jobackGrpList();
};

class jobackLibrary {
 public:                                         // --- data members ---
  jobackGroupValues values;
  jobackAliases aliases;
 public:                                     // --- member functions ---
  jobackLibrary();
  void clear();
  int loadFrom(string flname, libPath &libp);
  int findGroup(string gnam, jobackValues &jv);
  int estimate(jobackGrpList &jl, double &eps, double &sig);
};

#endif
