/* -------------------------------------------------------------------------
     "mol2smiles.cc" - covert KUCRS-mol to SMILES format
        This is a part of KUCRS:
           Knowledge-basing Utilities for Complex Reaction Systems
                        (c) 2002-2021 by A. Miyoshi, Hiroshima University
                                       created: Jan.  26, 2011
                                   last edited: Aug.  18, 2021
 --------------------------------------------------------------------------
 usage: mol2smiles MOLinput SMILESoutput [-l]
------------------------------------------------------------------------- */

#include "molecule.h"
using namespace std;

/*===== main routien =====================================================*/

int main(int argc, char *argv[]) {
  long mtyp;
  int outCodeList = false;
  string cdate, chemForm, typ, strSMILES;
  string clopt = "", fnOut, fnBas, fnLst, fnCsv;
  ofstream ols, ocs;
  groupLib grp;
  typeMolec tym;
  vector<string> vs;
  molecule spc;

  if ((argc < 3) || (argc > 4))
    { cout << "usage: mol2smiles MOLinput SMILESoutput [-l]\n"; return 1; }
  if (argc == 4) {
    clopt = argv[3];
    if (clopt == "-l") { outCodeList = true; }
    else
     { cout << "usage: mol2smiles MOLinput SMILESoutput [-l]\n"; return 1; }
  }
  fnOut = argv[2]; fnBas = getBaseName(fnOut); fnLst = fnBas + "_sp_.lst";
  fnCsv = fnBas + "_sp_.csv";
  cdate = "(" + strCurDate() + ")";
  cout << "--- Program mol2smiles " << cdate << " ---\n";
  ifstream ifs(argv[1]);
  if (!ifs) { cout << "failed to open [" << argv[1] << "].\n"; return 1; }
  ofstream ofs(argv[2]);
  if (!ofs) { cout << "failed to open [" << argv[2] << "].\n"; return 1; }
  if (outCodeList) {
    ols.open(fnLst.c_str());
    if (!ols) { cout << "failed to open [" << fnLst << "].\n"; return 1; }
    ocs.open(fnCsv.c_str());
    if (!ocs) { cout << "failed to open [" << fnCsv << "].\n"; return 1; }
  }


  while (spc.restoreFrom(ifs) == NORM) {
    if (!spc.isUnique) { spc.beUnique(); }
    spc.getUniqueView(vs);
    chemForm = spc.molec.getChemFormula();
    cout << "-----------------------------------------------------------\n";
    cout << " name:    " << spc.molName << endl;
    cout << " formula: " << chemForm << endl;
    strSMILES = spc.getSMILES();
    cout << " SMILES:  " << strSMILES << endl;
    cout << " struct:\n";
    cout << vs;
    cout << " groups: " << spc.getThermInput(grp) << endl;
    mtyp = spc.getType(grp); typ = tym.typNameExt(mtyp);
    cout << " code:    " << hex << mtyp << dec;
    if (typ != "") { cout << " (" << typ << ")"; }
    cout << endl;
    ofs << formatLeft(spc.molName, 18) << "  " << strSMILES << endl;
    if (outCodeList) {
      ols << spc.molName << " " << hex << mtyp << dec << " ";
      if (typ == "") { ols << spc.molName << endl; }
      else { ols << typ << endl; }
      ocs << "\"" << spc.molName << "\"," << "\"" << strSMILES << "\"\n";
    }
  }
  if (outCodeList) { ols.close(); }
  ofs.close();
  ifs.close();
  return 0;
}

